use crate::cli::FormatMode;
use crate::jira_config::JiraConfig;
use crate::renderer::{hyperlink, render_doc};
use crossterm::style::{Color, Stylize};
use crossterm::terminal;
use libjirac::client::commands::IssueGetCommand;
use libjirac::client::JiraClient;
use libjirac::entities::issue::{Description, JiraIssue};
use std::io::Write;

fn print_header(title: &str, columns: usize) {
    let padding = columns - (title.len() + 4);
    println!("\n== {} {:=<width$}", title, "", width = padding);
}

fn pretty_print(config: &JiraConfig, issue: &JiraIssue) -> Result<(), Box<dyn std::error::Error>> {
    let (columns, _) = terminal::size()?;

    print_header("Title", columns.into());
    println!(
        "{}: {}",
        issue.key.clone().green(),
        issue.fields.summary.clone().bold().green()
    );

    print_header("Details", columns.into());
    let mut tw = tabwriter::TabWriter::new(vec![]);
    writeln!(tw, "{}:\t{}", "Priority".blue(), issue.fields.priority.name)?;
    writeln!(tw, "{}:\t{}", "Status".blue(), issue.fields.status.name)?;
    writeln!(
        tw,
        "{}:\t{} <{}>",
        "Assignee".blue(),
        issue.fields.assignee.display_name,
        issue
            .fields
            .assignee
            .email_address
            .clone()
            .unwrap_or("invalid@example.com".to_string())
    )?;
    writeln!(
        tw,
        "{}:\t{} <{}>",
        "Reporter".blue(),
        issue.fields.reporter.display_name,
        issue
            .fields
            .reporter
            .email_address
            .clone()
            .unwrap_or("invalid@example.com".to_string())
    )?;
    writeln!(
        tw,
        "{}:\t{}",
        "Created".blue(),
        issue.fields.created.with_timezone(&chrono::Local)
    )?;
    writeln!(
        tw,
        "{}:\t{}",
        "Due Date".blue(),
        match issue.fields.due_date {
            None => "None".to_string(),
            Some(x) => x.to_string(),
        }
    )?;

    tw.flush().unwrap();

    let written = String::from_utf8(tw.into_inner().unwrap()).unwrap();
    print!("{}", written);
    print_header("Description", columns.into());
    match issue.fields.description.clone() {
        Some(x) => match x {
            Description::Doc(doc) => render_doc(doc)?,
        },
        None => println!("(Issue does not have a description)"),
    }

    print_header("Comments", columns.into());
    for comment in issue.fields.comment.clone().unwrap_or_default().comments {
        println!(
            "{} at {}",
            comment.author.display_name.red(),
            comment.created.with_timezone(&chrono::Local)
        );
        render_doc(comment.body)?;
    }

    print_header("Actions", columns.into());
    println!(
        "{}",
        hyperlink(
            &format!("{}/browse/{}", config.url, issue.key),
            &"Open Issue".green().underline(Color::Green).to_string()
        )
    );

    Ok(())
}

pub fn json_print(issues: &JiraIssue) -> Result<(), Box<dyn std::error::Error>> {
    let j = serde_json::to_string_pretty(issues)?;
    println!("{}", j);
    Ok(())
}

pub async fn exec(output: FormatMode, issue_key: &str) -> Result<(), Box<dyn std::error::Error>> {
    let config = JiraConfig::load().map_err(|e| format!("Configuration error: {}", e))?;
    if output != FormatMode::Json {
        println!("Loading issue data");
    }

    let client = JiraClient::from(&config);

    let get_cmd = IssueGetCommand::new(issue_key);
    let fetched_issue = client.exec(get_cmd).await?;

    match output {
        FormatMode::Pretty => pretty_print(&config, &fetched_issue)?,
        FormatMode::Json => json_print(&fetched_issue)?,
        FormatMode::Compact => todo!(),
    }

    Ok(())
}
